@@ -697,12 +697,23 @@ def test_normal_with_slightly_nonplanar():
         geo_dict = json.load(fp)
     face_geo = Face3D.from_dict(geo_dict)
 
-    assert -0.99 > face_geo.normal.z > -1.01
+    assert 0.99 < face_geo.normal.z < 1.01
     assert face_geo.check_planar(0.000001, False) is False
     with pytest.raises(Exception):
         face_geo.check_planar(0.000001)
 
 
+def test_normal_with_colinear_vertices():
+    """Test that shapes with colinear vertices have a relatively close normal."""
+    geo_file = './tests/json/faces_colinear_verts.json'
+    with open(geo_file, 'r') as fp:
+        geo_dict = json.load(fp)
+    face_geos = [Face3D.from_dict(geo) for geo in geo_dict]
+
+    assert -0.01 < face_geos[0].normal.z < 0.01
+    assert -0.01 < face_geos[1].normal.z < 0.01
+
+
 def test_flip():
     """Test the flip method of Face3D."""
     pts_1 = (Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 2, 2), Point3D(0, 2, 2))
@@ -1008,94 +1019,94 @@ def test_mesh_grid():
     assert mesh_3.max.z == pytest.approx(-1, rel=1e-2)
 
 
-def test_countour_by_number():
-    """Test the countour_by_number method."""
+def test_contour_by_number():
+    """Test the contour_by_number method."""
     pts_1 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 0, 0)]
     pts_2 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(4, 0, 0)]
     plane = Plane(Vector3D(0, 1, 0))
     face_1 = Face3D(pts_1, plane)
     face_2 = Face3D(pts_2, plane)
 
-    contours = face_1.countour_by_number(4, Vector2D(0, 1), False, 0.01)
+    contours = face_1.contour_by_number(4, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
     assert contours[0].p2.z == pytest.approx(2, rel=1e-3)
     assert contours[-1].p2.z == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_1.countour_by_number(4, Vector2D(1), False, 0.01)
+    contours = face_1.contour_by_number(4, Vector2D(1), False, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(1.5, rel=1e-3)
 
-    contours = face_1.countour_by_number(4, Vector2D(1), True, 0.01)
+    contours = face_1.contour_by_number(4, Vector2D(1), True, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_2.countour_by_number(4, Vector2D(0, 1), False, 0.01)
+    contours = face_2.contour_by_number(4, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
-    contours = face_2.countour_by_number(8, Vector2D(1), False, 0.01)
+    contours = face_2.contour_by_number(8, Vector2D(1), False, 0.01)
     assert len(contours) == 8
 
 
-def test_countour_by_distance_between():
-    """Test the countour_by_distance_between method."""
+def test_contour_by_distance_between():
+    """Test the contour_by_distance_between method."""
     pts_1 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 0, 0)]
     pts_2 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(4, 0, 0)]
     plane = Plane(Vector3D(0, 1, 0))
     face_1 = Face3D(pts_1, plane)
     face_2 = Face3D(pts_2, plane)
 
-    contours = face_1.countour_by_distance_between(0.5, Vector2D(0, 1), False, 0.01)
+    contours = face_1.contour_by_distance_between(0.5, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
     assert contours[0].p2.z == pytest.approx(2, rel=1e-3)
     assert contours[-1].p2.z == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_1.countour_by_distance_between(0.5, Vector2D(1), False, 0.01)
+    contours = face_1.contour_by_distance_between(0.5, Vector2D(1), False, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(1.5, rel=1e-3)
 
-    contours = face_1.countour_by_distance_between(0.5, Vector2D(1), True, 0.01)
+    contours = face_1.contour_by_distance_between(0.5, Vector2D(1), True, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_2.countour_by_distance_between(0.5, Vector2D(0, 1), False, 0.01)
+    contours = face_2.contour_by_distance_between(0.5, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
-    contours = face_2.countour_by_distance_between(0.5, Vector2D(1), False, 0.01)
+    contours = face_2.contour_by_distance_between(0.5, Vector2D(1), False, 0.01)
     assert len(contours) == 8
 
 
-def test_countour_fins_by_number():
-    """Test the countour_fins_by_number method."""
+def test_contour_fins_by_number():
+    """Test the contour_fins_by_number method."""
     pts_1 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 0, 0)]
     plane = Plane(Vector3D(0, 1, 0))
     face_1 = Face3D(pts_1, plane)
 
-    fins = face_1.countour_fins_by_number(4, 0.5, 0.5, 0, Vector2D(0, 1), False, 0.01)
+    fins = face_1.contour_fins_by_number(4, 0.5, 0.5, 0, Vector2D(0, 1), False, 0.01)
     assert len(fins) == 4
 
-    fins = face_1.countour_fins_by_number(4, 0.5, 0.5, 0, Vector2D(1), False, 0.01)
+    fins = face_1.contour_fins_by_number(4, 0.5, 0.5, 0, Vector2D(1), False, 0.01)
     assert len(fins) == 4
 
-    fins = face_1.countour_fins_by_number(
+    fins = face_1.contour_fins_by_number(
         4, 0.5, 0.5, math.pi/4, Vector2D(0, 1), False, 0.01)
     assert len(fins) == 4
 
 
-def test_countour_fins_by_distance_between():
-    """Test the countour_fins_by_distance_between method."""
+def test_contour_fins_by_distance_between():
+    """Test the contour_fins_by_distance_between method."""
     pts_1 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(2, 0, 0)]
     pts_2 = [Point3D(0, 0, 0), Point3D(0, 0, 2), Point3D(2, 0, 2), Point3D(4, 0, 0)]
     plane = Plane(Vector3D(0, 1, 0))
     face_1 = Face3D(pts_1, plane)
     face_2 = Face3D(pts_2, plane)
 
-    fins = face_1.countour_fins_by_distance_between(
+    fins = face_1.contour_fins_by_distance_between(
         0.5, 0.5, 0.5, 0, Vector2D(0, 1), False, 0.01)
     assert len(fins) == 4
 
-    fins = face_1.countour_fins_by_distance_between(
+    fins = face_1.contour_fins_by_distance_between(
         0.25, 0.5, 0.5,0, Vector2D(1), False, 0.01)
     assert len(fins) == 8
 
-    fins = face_2.countour_fins_by_distance_between(
+    fins = face_2.contour_fins_by_distance_between(
         0.5, 0.5, 0.5, 0, Vector2D(1), False, 0.01)
     assert len(fins) == 8
 
